<?php

namespace Botble\Author\Http\Controllers;

use Botble\Base\Events\BeforeEditContentEvent;
use Botble\Author\Http\Requests\AuthorRequest;
use Botble\Author\Repositories\Interfaces\AuthorInterface;
use Botble\Base\Http\Controllers\BaseController;
use Illuminate\Http\Request;
use Exception;
use Botble\Author\Tables\AuthorTable;
use Botble\Base\Events\CreatedContentEvent;
use Botble\Base\Events\DeletedContentEvent;
use Botble\Base\Events\UpdatedContentEvent;
use Botble\Base\Http\Responses\BaseHttpResponse;
use Botble\Author\Forms\AuthorForm;
use Botble\Base\Forms\FormBuilder;

class AuthorController extends BaseController
{
    /**
     * @var AuthorInterface
     */
    protected $authorRepository;

    /**
     * @param AuthorInterface $authorRepository
     */
    public function __construct(AuthorInterface $authorRepository)
    {
        $this->authorRepository = $authorRepository;
    }

    /**
     * @param AuthorTable $table
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     * @throws \Throwable
     */
    public function index(AuthorTable $table)
    {
        page_title()->setTitle(trans('plugins/author::author.name'));

        return $table->renderTable();
    }

    /**
     * @param FormBuilder $formBuilder
     * @return string
     */
    public function create(FormBuilder $formBuilder)
    {
        page_title()->setTitle(trans('plugins/author::author.create'));

        return $formBuilder->create(AuthorForm::class)->renderForm();
    }

    /**
     * @param AuthorRequest $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     */
    public function store(AuthorRequest $request, BaseHttpResponse $response)
    {
        $author = $this->authorRepository->createOrUpdate($request->input());

        event(new CreatedContentEvent(AUTHOR_MODULE_SCREEN_NAME, $request, $author));

        return $response
            ->setPreviousUrl(route('author.index'))
            ->setNextUrl(route('author.edit', $author->id))
            ->setMessage(trans('core/base::notices.create_success_message'));
    }

    /**
     * @param $id
     * @param Request $request
     * @param FormBuilder $formBuilder
     * @return string
     */
    public function edit($id, FormBuilder $formBuilder, Request $request)
    {
        $author = $this->authorRepository->findOrFail($id);

        event(new BeforeEditContentEvent($request, $author));

        page_title()->setTitle(trans('plugins/author::author.edit') . ' "' . $author->getFullName() . '"');

        return $formBuilder->create(AuthorForm::class, ['model' => $author])->renderForm();
    }

    /**
     * @param $id
     * @param AuthorRequest $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     */
    public function update($id, AuthorRequest $request, BaseHttpResponse $response)
    {
        $author = $this->authorRepository->findOrFail($id);

        $author->fill($request->input());

        $this->authorRepository->createOrUpdate($author);

        event(new UpdatedContentEvent(AUTHOR_MODULE_SCREEN_NAME, $request, $author));

        return $response
            ->setPreviousUrl(route('author.index'))
            ->setMessage(trans('core/base::notices.update_success_message'));
    }

    /**
     * @param $id
     * @param Request $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     */
    public function destroy(Request $request, $id, BaseHttpResponse $response)
    {
        try {
            $author = $this->authorRepository->findOrFail($id);

            $this->authorRepository->delete($author);

            event(new DeletedContentEvent(AUTHOR_MODULE_SCREEN_NAME, $request, $author));

            return $response->setMessage(trans('core/base::notices.delete_success_message'));
        } catch (Exception $exception) {
            return $response
                ->setError()
                ->setMessage($exception->getMessage());
        }
    }

    /**
     * @param Request $request
     * @param BaseHttpResponse $response
     * @return BaseHttpResponse
     * @throws Exception
     */
    public function deletes(Request $request, BaseHttpResponse $response)
    {
        $ids = $request->input('ids');
        if (empty($ids)) {
            return $response
                ->setError()
                ->setMessage(trans('core/base::notices.no_select'));
        }

        foreach ($ids as $id) {
            $author = $this->authorRepository->findOrFail($id);
            $this->authorRepository->delete($author);
            event(new DeletedContentEvent(AUTHOR_MODULE_SCREEN_NAME, $request, $author));
        }

        return $response->setMessage(trans('core/base::notices.delete_success_message'));
    }
}
