<?php

namespace Botble\Author\Providers;

use Botble\Author\Models\Author;
use Botble\Base\Enums\BaseStatusEnum;
use Botble\Blog\Models\Post;
use Illuminate\Support\ServiceProvider;
use Botble\Author\Repositories\Caches\AuthorCacheDecorator;
use Botble\Author\Repositories\Eloquent\AuthorRepository;
use Botble\Author\Repositories\Interfaces\AuthorInterface;
use Botble\Base\Supports\Helper;
use Event;
use Botble\Base\Traits\LoadAndPublishDataTrait;
use Illuminate\Routing\Events\RouteMatched;
use SeoHelper;
use SlugHelper;
use Botble\Author\Http\Requests\CustomPostRequest;

class AuthorServiceProvider extends ServiceProvider
{
    use LoadAndPublishDataTrait;

    public function register()
    {
        $this->app->bind(AuthorInterface::class, function () {
            return new AuthorCacheDecorator(new AuthorRepository(new Author));
        });

        Helper::autoload(__DIR__ . '/../../helpers');
    }

    public function boot()
    {
        SlugHelper::registerModule(Author::class, 'Authors');
        SlugHelper::setPrefix(Author::class, 'author');

        $this->setNamespace('plugins/author')
            ->loadAndPublishConfigurations(['permissions'])
            ->loadMigrations()
            ->loadAndPublishTranslations()
            ->loadRoutes(['web'])
            ->loadAndPublishViews();

        Event::listen(RouteMatched::class, function () {
            dashboard_menu()->registerItem([
                'id'          => 'cms-plugins-author',
                'priority'    => 5,
                'parent_id'   => null,
                'name'        => 'plugins/author::author.name',
                'icon'        => 'fa fa-user-secret',
                'url'         => route('author.index'),
                'permissions' => ['author.index'],
            ]);
        });

        $this->app->booted(function () {
            SeoHelper::registerModule(Author::class);

            if (is_plugin_active('blog')) {
                add_filter(BASE_FILTER_BEFORE_RENDER_FORM, function ($form, $data) {
                    if (get_class($data) == Post::class) {
                        $authors = $this->app->make(AuthorInterface::class)
                            ->allBy(['status' => BaseStatusEnum::PUBLISHED]);

                        $authorsArray = [];
                        foreach ($authors as $author) {
                            $authorsArray[$author->id] = $author->getFullName();
                        }

                        $form
                            ->setValidatorClass(CustomPostRequest::class)
                            ->addAfter('status', 'author_id', 'customSelect', [
                                'label'      => __('Author'),
                                'label_attr' => ['class' => 'control-label required'],
                                'attr'       => [
                                    'placeholder' => __('Select an author...'),
                                ],
                                'choices'    => $authorsArray,
                            ]);
                    }

                    return $form;
                }, 127, 2);

                add_action(BASE_ACTION_AFTER_CREATE_CONTENT, function ($type, $request, $object) {
                    if (in_array($type, [POST_MODULE_SCREEN_NAME])) {
                        $object->author_id = $request->input('author_id');
                        $object->author_type = Author::class;
                        $object->save();
                    }
                }, 123, 3);

                add_action(BASE_ACTION_AFTER_UPDATE_CONTENT, function ($type, $request, $object) {
                    if (in_array($type, [POST_MODULE_SCREEN_NAME])) {
                        $object->author_id = $request->input('author_id');
                        $object->author_type = Author::class;
                        $object->save();
                    }
                }, 123, 3);
            }
        });

        $this->app->register(EventServiceProvider::class);
    }
}
