<?php

namespace Botble\Career\Tables;

use BaseHelper;
use Botble\Base\Enums\BaseStatusEnum;
use Botble\Career\Exports\PostExport;
use Botble\Career\Repositories\Interfaces\CategoryInterface;
use Botble\Career\Repositories\Interfaces\PostInterface;
use Botble\Table\Abstracts\TableAbstract;
use Html;
use Illuminate\Contracts\Routing\UrlGenerator;
use Illuminate\Support\Facades\Auth;
use Yajra\DataTables\DataTables;
use DB;

class CareerTable extends TableAbstract
{
    /**
     * @var bool
     */
    protected $hasActions = true;

    /**
     * @var bool
     */
    protected $hasFilter = false;

    /**
     * @var CategoryInterface
     */
    protected $categoryRepository;

    /**
     * @var string
     */
    protected $exportClass = PostExport::class;

    /**
     * @var int
     */
    protected $defaultSortColumn = 6;

    /**
     * PostTable constructor.
     * @param DataTables $table
     * @param UrlGenerator $urlGenerator
     * @param PostInterface $postRepository
     * @param CategoryInterface $categoryRepository
     */
    public function __construct(
        DataTables $table,
        UrlGenerator $urlGenerator,
        PostInterface $postRepository,
        CategoryInterface $categoryRepository
    ) {
        parent::__construct($table, $urlGenerator);

        $this->repository = $postRepository;
        $this->categoryRepository = $categoryRepository;

        if (!Auth::user()->hasAnyPermission(['careerposts.edit', 'careerposts.destroy'])) {
            $this->hasOperations = false;
            $this->hasActions = false;
        }
    }

    /**
     * {@inheritDoc}
     */
    public function ajax()
    {
        $data = $this->table
            ->eloquent($this->query())
            ->editColumn('f_name', function ($item) {
                if (!Auth::user()->hasPermission('careerposts.edit')) {
                    return BaseHelper::clean($item->f_name);
                }

                return Html::link(route('careerposts.edit', $item->id), BaseHelper::clean($item->f_name));
            })
            ->editColumn('checkbox', function ($item) {
                return $this->getCheckbox($item->id);
            })
            ->editColumn('phone', function ($item) {
                return BaseHelper::clean($item->phone);
            })
            ->addColumn('operations', function ($item) {
                return $this->getOperations('careerposts.edit', 'careerposts.destroy', $item);
            });

        return $this->toJson($data);
    }

    /**
     * {@inheritDoc}
     */
    public function query()
    {
        /* $query = $this->repository->getModel()
            ->with([
                'CareersJobs'
            ])
            ->select([
                'id',
                'f_name',
                'email',
                'phone',
                'csal',
                'esal',
                'period'
            ])->orderBy('id', 'asc'); */
            $query = $this->repository->getModel()->select('careers.*','careers_jobs.name AS jobname')
            ->leftJoin('careers_jobs', function($leftJoin)
            {
                $leftJoin->on('careers.job_id', '=', 'careers_jobs.id');
            })->orderBy('id', 'desc');;
            // $query = DB::select('SELECT careers.*, careers_jobs.name
            // FROM careers
            // LEFT JOIN careers_jobs ON careers.job_id = careers_jobs.id
            // ORDER BY careers.id');
        return $query;
        //return $this->applyScopes($query);
    }

    /**
     * {@inheritDoc}
     */
    public function columns()
    {
        return [
            'id'         => [
                'title' => trans('core/base::tables.id'),
                'width' => '20px',
            ],
            'f_name'      => [
                'title' => trans('core/base::tables.f_name'),
                'width' => '70px',
            ],
            'jobname'      => [
                'title' => trans('core/base::tables.job_name'),
                'width' => '70px',
                'orderable' => false
            ],
            'email'       => [
                'title' => trans('core/base::tables.email'),
                'class' => 'text-start',
            ],
            'phone'       => [
                'title' => trans('core/base::tables.phone'),
                'class' => 'text-start',
            ],
            'csal'       => [
                'title' => trans('core/base::tables.csal'),
                'class' => 'text-start',
            ],
            'esal'       => [
                'title' => trans('core/base::tables.esal'),
                'class' => 'text-start',
            ],
            'period'       => [
                'title' => trans('core/base::tables.period'),
                'class' => 'text-start',
            ]
        ];
    }

    /**
     * {@inheritDoc}
     */
    public function buttons()
    {
        //return $this->addCreateButton(route('careerposts.create'), 'careerposts.create');
    }

    /**
     * {@inheritDoc}
     */
    public function bulkActions(): array
    {
        return $this->addDeleteAction(route('careerposts.deletes'), 'careerposts.destroy', parent::bulkActions());
    }

    /**
     * {@inheritDoc}
     */
    // public function getBulkChanges(): array
    // {
    //     return [
    //         'name'       => [
    //             'title'    => trans('core/base::tables.name'),
    //             'type'     => 'text',
    //             'validate' => 'required|max:120',
    //         ],
    //         'status'     => [
    //             'title'    => trans('core/base::tables.status'),
    //             'type'     => 'customSelect',
    //             'choices'  => BaseStatusEnum::labels(),
    //             'validate' => 'required|in:' . implode(',', BaseStatusEnum::values()),
    //         ],
    //         'category'         => [
    //             'title'    => trans('plugins/career::posts.category'),
    //             'type'     => 'select-search',
    //             'validate' => 'required',
    //             'callback' => 'getCategories',
    //         ],
    //         'created_at' => [
    //             'title'    => trans('core/base::tables.created_at'),
    //             'type'     => 'date',
    //             'validate' => 'required',
    //         ],
    //     ];
    // }

    /**
     * @return array
     */
    public function getCategories(): array
    {
        return $this->categoryRepository->pluck('name', 'id');
    }

    /**
     * {@inheritDoc}
     */
    // public function applyFilterCondition($query, string $key, string $operator, ?string $value)
    // {
    //     switch ($key) {
    //         case 'created_at':
    //             if (!$value) {
    //                 break;
    //             }

    //             $value = BaseHelper::formatDate($value);

    //             return $query->whereDate($key, $operator, $value);
    //         case 'category':
    //             if (!$value) {
    //                 break;
    //             }

    //             if (!BaseHelper::isJoined($query, 'post_categories')) {
    //                 $query = $query
    //                     ->join('post_categories', 'post_categories.post_id', '=', 'posts.id')
    //                     ->join('categories', 'post_categories.category_id', '=', 'categories.id')
    //                     ->select($query->getModel()->getTable() . '.*');
    //             }

    //             return $query->where('post_categories.category_id', $value);
    //     }

    //     return parent::applyFilterCondition($query, $key, $operator, $value);
    // }

    /**
     * {@inheritDoc}
     */
    public function saveBulkChangeItem($item, string $inputKey, ?string $inputValue)
    {
        if ($inputKey === 'category') {
            $item->categories()->sync([$inputValue]);

            return $item;
        }

        return parent::saveBulkChangeItem($item, $inputKey, $inputValue);
    }

    /**
     * {@inheritDoc}
     */
    public function getDefaultButtons(): array
    {
        return [
            'export',
            'reload',
        ];
    }
}
